/* __placeholder__ */
import { deepCloneJSON, get, set } from '@codeperate/utils';
import { computed, provide, ref, watch } from 'vue';
import { debounce } from './utils/debounce.utils';
import { LocalStorage } from './utils/localstorage.util';
const { defineProps, defineSlots, defineEmits, defineExpose, defineModel, defineOptions, withDefaults, } = await import('vue');
let __VLS_typeProps;
const props = withDefaults(defineProps(), {
    view: false,
    saveExpiry: 1000 * 60 * 60 * 24,
    autoSave: false,
    saveDelay: 5000,
});
const emit = defineEmits();
const widgetMap = new Map();
const widgetCount = ref(0);
const formValue = ref(deepCloneJSON(props.value));
watch(() => props.value, () => (formValue.value = deepCloneJSON(props.value)));
const validate = (option = {}) => {
    const defaultOption = {
        scrollToInvalidWidget: true,
    };
    Object.assign(option, defaultOption);
    const result = getWidgets()
        .filter(w => !w.isHidden())
        .map(w => w.validate())
        .filter(r => r);
    if (option.scrollToInvalidWidget) {
        let invalidField = result.find(r => r?.validity == false);
        if (invalidField) {
            getWidgets(invalidField.path)[0].el?.value?.scrollIntoView({
                behavior: 'smooth',
                block: 'center',
                inline: 'center',
            });
        }
    }
    return result;
};
const undoValidate = (path = '') => {
    const widgets = getWidgets(path);
    widgets.forEach(w => w.undoValidate());
};
const regWidget = (path, widget) => {
    const widgets = widgetMap.get(path);
    if (!widgets) {
        widgetMap.set(path, [widget]);
        widgetCount.value += 1;
    }
    else {
        const found = widgets.find(w => w === widget);
        if (found)
            return;
        else {
            widgets.push(widget);
            widgetCount.value += 1;
        }
    }
};
const unRegWidget = (path, widget) => {
    const widgets = widgetMap.get(path);
    if (!widgets)
        return;
    const found = widgets.find(w => w === widget);
    if (found)
        widgetMap.set(path, widgets.filter(w => w !== widget));
};
const getWidgets = (path = '') => {
    const pathStr = typeof path == 'string' ? path : path.join('.');
    const widgets = [];
    for (const [key, value] of widgetMap.entries()) {
        if (key.startsWith(pathStr))
            widgets.push(...value);
    }
    return widgets;
};
const getView = () => {
    return props.view;
};
const getValue = (path) => {
    return get(formValue.value, path);
};
const setValue = (path = '', value) => {
    if (typeof path == 'string')
        path = path.split('.').filter(p => p);
    if (path.length == 0) {
        formValue.value = value;
    }
    else {
        if (formValue.value == undefined) {
            if (!isNaN(Number(path[0])))
                formValue.value = [];
            else
                formValue.value = {};
        }
        set(formValue.value, path, value);
    }
    emit('change', formValue.value);
    emit('update:value', formValue.value);
    if (props.autoSave)
        delayedSave();
};
const setValueByFn = (fn) => {
    formValue.value = fn(formValue);
};
const getSchema = (path) => {
    if (typeof path == 'string')
        path = path.split('.').filter(p => p);
    let curPos = props.schema;
    if (!path)
        return curPos;
    for (const p of path) {
        if (curPos?.properties && p in curPos.properties)
            curPos = curPos.properties[p];
        else if (curPos.items)
            curPos = curPos.items;
        else
            return {};
    }
    return curPos;
};
const exportValue = async (options = {}) => {
    const defaultOptions = {
        removeAdditional: true,
    };
    options = Object.assign(defaultOptions, options);
    let wrappedValue = { value: deepCloneJSON(formValue.value) };
    const widgets = getWidgets();
    const promises = [];
    for (const widget of widgets)
        promises.push(widget.onExportValue(wrappedValue, options));
    await Promise.all(promises);
    return wrappedValue.value;
};
const save = async () => {
    if (!props.saveLocation)
        throw new Error('You must define save location first.');
    try {
        const value = deepCloneJSON(getValue());
        const widgets = getWidgets();
        const promises = [];
        for (const widget of widgets)
            promises.push(widget.onSaveHistory(value));
        await Promise.all(promises);
        LocalStorage.set(props.saveLocation, {
            value,
            expiryAt: new Date().getTime() + props.saveExpiry,
        });
    }
    catch (err) { }
};
const delayedSave = debounce(save, props.saveDelay);
const load = async () => {
    if (!props.saveLocation)
        throw new Error('You must define save location first.');
    const history = LocalStorage.get(props.saveLocation);
    const widgets = getWidgets();
    const promises = [];
    for (const widget of widgets)
        promises.push(widget.onLoadHistory(history.value));
    await Promise.all(promises);
    setValue('', history.value);
};
const exposed = {
    formValue,
    exportValue,
    regWidget,
    unRegWidget,
    getWidgets,
    getValue,
    setValue,
    setValueByFn,
    getSchema,
    getView,
    validate,
    undoValidate,
    save,
    load,
};
const componentProps = computed(() => {
    let _props = props.schema.props ?? {};
    return typeof _props == 'function' ? _props({ form: exposed }) : _props;
});
provide('formBuilder', exposed);
const __VLS_exposed = exposed;
defineExpose(exposed);
// onRenderTriggered(({ key, target, type }) => {
//     console.log(`Render triggered on component`, { key, target, type });
// });
const __VLS_withDefaultsArg = (function (t) { return t; })({
    view: false,
    saveExpiry: 1000 * 60 * 60 * 24,
    autoSave: false,
    saveDelay: 5000,
});
const __VLS_fnComponent = (await import('vue')).defineComponent({
    emits: {},
});
let __VLS_functionalComponentProps;
function __VLS_template() {
    let __VLS_ctx;
    /* Components */
    let __VLS_otherComponents;
    let __VLS_own;
    let __VLS_localComponents;
    let __VLS_components;
    let __VLS_styleScopedClasses;
    let __VLS_resolvedLocalAndGlobalComponents;
    const __VLS_0 = (props.schema.widget?.component);
    // @ts-ignore
    const __VLS_1 = __VLS_asFunctionalComponent(__VLS_0, new __VLS_0({ path: (""), ...(__VLS_ctx.componentProps), }));
    const __VLS_2 = __VLS_1({ path: (""), ...(__VLS_ctx.componentProps), }, ...__VLS_functionalComponentArgsRest(__VLS_1));
    ({}({ path: (""), ...(__VLS_ctx.componentProps), }));
    // @ts-ignore
    [componentProps,];
    const __VLS_5 = __VLS_pickFunctionalComponentCtx(__VLS_0, __VLS_2);
    if (typeof __VLS_styleScopedClasses === 'object' && !Array.isArray(__VLS_styleScopedClasses)) {
    }
    var __VLS_slots;
    return __VLS_slots;
    const __VLS_componentsOption = {};
    let __VLS_name;
    let __VLS_defineComponent;
    const __VLS_internalComponent = __VLS_defineComponent({
        setup() {
            return {
                componentProps: componentProps,
            };
        },
        props: {},
        emits: {},
    });
}
export default (await import('vue')).defineComponent({
    setup() {
        return {
            ...__VLS_exposed,
        };
    },
    props: {},
    emits: {},
});
;
