import { ComputedRef, Ref } from 'vue';
import { ExportValueOptions, FormBuilderContext, FormSchema, ValidatedMeta } from '../FormBuilder.interface';
export interface FormWidgetProps {
    path: string;
    targetPath?: string;
    required?: boolean;
    validate?: boolean;
    hidden?: boolean;
}
export interface IFormWidget<V = any> {
    isValidated: Ref<boolean>;
    form: FormBuilderContext;
    value: ComputedRef<V>;
    validatedMeta: Ref<ValidatedMeta>;
    view: Ref<boolean>;
    schema: ComputedRef<FormSchema>;
    el?: Ref<HTMLElement | undefined>;
    isRequired: () => boolean;
    isValid: () => boolean | undefined;
    isHidden: () => boolean;
    getSchema: () => FormSchema;
    getPath: () => string;
    getPathArr: () => string[];
    getValue: () => V;
    setValue: (value: any) => void;
    register: (exposed?: Partial<IFormWidget<V>> & {
        [key: string]: any;
    }) => IFormWidget;
    validator: () => void;
    validate: () => ValidatedMeta;
    undoValidate: () => void;
    onExportValue: (wrappedValue: {
        value: any;
    }, options: ExportValueOptions) => Promise<any>;
    onSaveHistory: (value: any) => Promise<void>;
    onLoadHistory: (value: any) => Promise<void>;
}
export declare function useFormWidget<V = any, N extends string = any>(name: N, props: FormWidgetProps): Omit<IFormWidget<V>, 'el'>;
